package com.softwarepearls.apps.hardware.raspberry.artekit.mrwood;

import com.pi4j.io.gpio.*;
import com.softwarepearls.apps.hardware.raspberry.artekit.shiftreg.SN74LV595;

/****************************************************************************
 * AK-MrWood-L. One or more 7-segment LED display(s) combined with Texas
 * Instruments 8-Bit Shift Register. Can be daisy-chained to form multi-digit
 * displays.
 * <P>
 * Requires same 3 control lines as a plain shift register (SER, SRCLK, RCLK).
 * <UL>
 * <LI>SER (the shifted-in bit, most significant bit first)
 * <LI>SRCLK (the clock signal ordering a bit to shift in)
 * <LI>RCLK (the value latch clock signal ordering the shifted bits to be put on
 * the output)
 * </UL>
 *****************************************************************************/
public final class MrWood {

	// The LED segment codes A-G are industry-standard (i.e. clockwise starting
	// from top horizontal)
	private static final String[] DIGIT_SEGMENTS_LUT = {
			"ABCDEF",// 0
			"BC",// 1
			"ABGED",// 2
			"ABGCD", // 3
			"FGBC", // 4
			"AFGCD", // 5
			"FEDCG", // 6
			"ABC", // 7
			"ABCDEFG", // 8
			"ABCFG", // 9
			"", // (blank)
	};

	private static final byte[] DIGIT_SEGMENT_BITS_LUT = new byte[DIGIT_SEGMENTS_LUT.length];

	private final SN74LV595 shiftRegister;

	private final int numDigits;

	static {
		initSegmentsLUT();
	}

	private static void initSegmentsLUT() {

		final int numDigits = DIGIT_SEGMENTS_LUT.length;
		for (int digit = 0; digit < numDigits; digit++) {
			final String segmentCodes = DIGIT_SEGMENTS_LUT[digit];
			final int numSegments = segmentCodes.length();
			byte segmentBits = 0;
			for (int segment = 0; segment < numSegments; segment++) {
				final char segmentCode = segmentCodes.charAt(segment);
				final int bitNumber = segmentCode - 'A';
				segmentBits |= 1 << bitNumber;
			}
			DIGIT_SEGMENT_BITS_LUT[digit] = segmentBits;
		}
	}

	/**
	 * Constructor. Use to control single digit display.
	 * 
	 * @param ser
	 * @param srclk
	 * @param rclk
	 */
	public MrWood(final GpioPinDigitalOutput ser, final GpioPinDigitalOutput srclk, final GpioPinDigitalOutput rclk) {

		this(1, ser, srclk, rclk);
	}

	/**
	 * Constructor.
	 * 
	 * @param numDigits
	 * @param ser
	 * @param srclk
	 * @param rclk
	 */
	public MrWood(final int numDigits, final GpioPinDigitalOutput ser, final GpioPinDigitalOutput srclk, final GpioPinDigitalOutput rclk) {

		this.numDigits = numDigits;
		shiftRegister = new SN74LV595(ser, srclk, rclk);
	}

	/**
	 * Display the integer value on the MrWood chain, right-aligned.
	 * 
	 * @param value any n-digit value (where n is the number of MrWood displays)
	 */
	public void show(final int value) {

		final String digits = Integer.toString(value);
		final int digitsLength = digits.length();
		if (digitsLength > numDigits) {
			throw new IllegalArgumentException("Your value, " + value + ", will not display in only " + numDigits + " digits.");
		}

		int accumulator = value;
		for (int i = 0; i < numDigits; i++) {
			shiftDigitOut(accumulator % 10);
			accumulator = accumulator / 10;
		}
		shiftRegister.latchBits();
	}

	public void shiftDigitOut(final int digit) {

		final byte segmentBits = DIGIT_SEGMENT_BITS_LUT[digit % 10];

		shiftRegister.shiftOutMostSignificantBitsFirst(segmentBits);
	}

	public static void delay(final long millis) {
		try {
			Thread.sleep(millis);
		}
		catch (final InterruptedException ignored) {
			;
		}
	}

	public static void main(final String[] args) {

		final GpioController gpioController = GpioFactory.getInstance();

		final GpioPinDigitalOutput ser = gpioController.provisionDigitalOutputPin(RaspiPin.GPIO_05);
		final GpioPinDigitalOutput srclk = gpioController.provisionDigitalOutputPin(RaspiPin.GPIO_06);
		final GpioPinDigitalOutput rclk = gpioController.provisionDigitalOutputPin(RaspiPin.GPIO_04);

		final MrWood display = new MrWood(5, ser, srclk, rclk);

		int start = 9999;
		long delay = 200;
		if (args.length > 0) {
			start = Integer.parseInt(args[0]);
			delay = Integer.parseInt(args[1]);
		}

		int counter = start;
		while (counter >= 0) {
			display.show(counter--);
			delay(delay);
		}
	}
}
